/*
 *	linux/fs/rbf/file.c
 *	RBF File System - file operations.
 *	
 *	Copyright (C) 2001 Andrew Cannon <ajc@gmx.net>
 * 	
 *  This file is subject to the terms and conditions of the
 *  GNU General Public License. See the file COPYING in the
 *  main directory of this archive for more details.
 */

#include <linux/fs.h>
#include <linux/rbf_fs.h>
#include <linux/pagemap.h>
#include "rbf_defs.h"

#undef DEBUG
#define DEBUG

#ifndef DEBUG
#undef DPRINTK
#define DPRINTK(x...)
#endif

struct file_operations rbf_page_file_operations = {
	.llseek = 	generic_file_llseek,
	.read =		do_sync_read,
	.aio_read =	generic_file_aio_read,
	.write =	do_sync_write,
	.aio_write =	generic_file_aio_write,
	.mmap =		generic_file_mmap,
	.release =	rbf_release_file
};

static int rbf_writepage(struct page *page, struct writeback_control *wbc)
{
	return block_write_full_page(page, rbf_get_block, wbc);
}

static int rbf_readpage(struct file *file, struct page *page)
{
	return block_read_full_page(page, rbf_get_block);
}

static int rbf_write_begin(struct file *file, struct address_space *mapping,
			loff_t pos, unsigned len, unsigned flags,
			struct page **pagep, void **fsdata)
{
	struct rbf_inode_info *rbf_inode = rbf_i(mapping->host);
	*pagep = NULL;
	return cont_write_begin(file, mapping, pos, len, flags, pagep, fsdata,
				rbf_get_block,
				&rbf_inode->mmu_private);
}

static sector_t rbf_bmap(struct address_space *mapping, sector_t block)
{
	return generic_block_bmap(mapping, block, rbf_get_block);
}

struct address_space_operations rbf_page_address_space_ops = {
        .readpage       = rbf_readpage,
        .writepage      = rbf_writepage,
        .sync_page      = block_sync_page,
	.write_begin	= rbf_write_begin,
	.write_end	= generic_write_end,
        .bmap           = rbf_bmap
};
                                                
